@@ -1,5 +1,5 @@
 /*
-	stdsoap2.h 2.8.4
+	stdsoap2.h 2.8.7
 
 	gSOAP runtime engine
 
@@ -51,6 +51,8 @@ A commercial use license is available from Genivia, Inc., contact@genivia.com
 --------------------------------------------------------------------------------
 */
 
+#define GSOAP_H_VERSION 20807
+
 #ifdef WITH_SOAPDEFS_H
 # include "soapdefs.h"		/* include user-defined stuff */
 #endif
@@ -211,6 +213,8 @@ A commercial use license is available from Genivia, Inc., contact@genivia.com
 #  define HAVE_STRRCHR
 #  define HAVE_STRTOD
 #  define HAVE_SSCANF
+#  define HAVE_STRTOD_L
+#  define HAVE_SPRINTF_L
 #  define HAVE_STRTOL
 #  define HAVE_STRTOUL
 #  define HAVE_SYS_TIMEB_H
@@ -528,7 +532,7 @@ A commercial use license is available from Genivia, Inc., contact@genivia.com
 # endif
 #endif
 
-/* native Win and HP-UX compilers don't like empty structs */
+/* native Win, HP-UX, and AIX compilers don't like empty structs */
 #if defined(WIN32) || defined(HP_UX) || defined(_AIX41) || defined(_AIX43) || defined(VXWORKS)
 # define WITH_NOEMPTYSTRUCT
 #endif
@@ -553,8 +557,11 @@ A commercial use license is available from Genivia, Inc., contact@genivia.com
 
 /* if we have xlocale.h we use it to avoid decimal point conversion issues */
 #ifdef WITH_C_LOCALE
-# include <locale.h>
-# include <xlocale.h>
+# ifdef WIN32
+#  include <locale.h>
+# else
+#  include <xlocale.h>
+# endif
 #else
 # undef HAVE_STRTOF_L
 # undef HAVE_STRTOD_L
@@ -562,7 +569,7 @@ A commercial use license is available from Genivia, Inc., contact@genivia.com
 # undef HAVE_SPRINTF_L
 #endif
 
-#ifdef TANDEM
+#ifdef TANDEM_NONSTOP
 # define SOAP_BUFLEN (32767)
 /*# define WITH_NOSTDLIB */ /* uncommment to remove stdlib dependences */
 # define WITH_NOIO      /* no IO dependences, e.g. remove TCP/IP */
@@ -1329,7 +1336,7 @@ typedef soap_int32 soap_mode;
 
 #define soap_check_state(soap) (!(soap) || ((soap)->state != SOAP_INIT && (soap)->state != SOAP_COPY))
 
-/* parts, states, and events */
+/* parts */
 
 #define SOAP_BEGIN		0
 #define SOAP_IN_ENVELOPE	2
@@ -1341,8 +1348,10 @@ typedef soap_int32 soap_mode;
 #define SOAP_END_ENVELOPE	8
 #define SOAP_END		9
 
-#define SOAP_SEC_BEGIN		10
-#define SOAP_SEC_SIGN		11
+/* events */
+
+#define SOAP_SEC_BEGIN		1
+#define SOAP_SEC_SIGN		2
 
 /* DEBUG macros */
 
@@ -1921,9 +1930,9 @@ struct SOAP_STD_API soap
   char arraySize[SOAP_TAGLEN];
   char arrayOffset[SOAP_TAGLEN];
   short other;
-  short position;
-  int positions[SOAP_MAXDIMS];
   short root;
+  int position;
+  int positions[SOAP_MAXDIMS];
   struct soap_attribute *attributes;	/* attribute list */
   short encoding;	/* when set, output encodingStyle */
   short mustUnderstand;	/* a mustUnderstand element was parsed or is output */
@@ -1931,6 +1940,7 @@ struct SOAP_STD_API soap
   short ns;		/* when not set, output full xmlns bindings */
   short part;		/* SOAP part state (header or body) */
   short event;		/* engine events and states for use by plugins */
+  unsigned int evlev;	/* event level */
   short alloced;
   short peeked;
   size_t chunksize;
@@ -2025,7 +2035,11 @@ struct SOAP_STD_API soap
   char session_host[SOAP_TAGLEN];
   int session_port;
 #ifdef WITH_C_LOCALE
+# ifdef WIN32
+  _locale_t c_locale;		/* set to C locale by default */
+# else
   locale_t c_locale;		/* set to C locale by default */
+# endif
 #else
   void *c_locale;
 #endif
@@ -2108,6 +2122,14 @@ soap_wchar soap_get0(struct soap*);
 soap_wchar soap_get1(struct soap*);
 #endif
 
+#define soap_init(soap) soap_init1(soap, SOAP_IO_DEFAULT)
+#define soap_init1(soap, mode) soap_init2(soap, mode, mode)
+#define soap_init2(soap, imode, omode) soap_init0(soap, GSOAP_H_VERSION, imode, omode)
+
+#define soap_new() soap_new1(SOAP_IO_DEFAULT)
+#define soap_new1(mode) soap_new2(mode, mode)
+#define soap_new2(imode, omode) soap_new0(GSOAP_H_VERSION, imode, omode)
+
 #define soap_revget1(soap) ((soap)->bufidx--)
 #define soap_unget(soap, c) ((soap)->ahead = c)
 #define soap_register_plugin(soap, plugin) soap_register_plugin_arg(soap, plugin, NULL)
@@ -2176,6 +2198,8 @@ SOAP_FMAC1 const char** SOAP_FMAC2 soap_faultcode(struct soap*);
 SOAP_FMAC1 const char** SOAP_FMAC2 soap_faultsubcode(struct soap*);
 SOAP_FMAC1 const char** SOAP_FMAC2 soap_faultstring(struct soap*);
 SOAP_FMAC1 const char** SOAP_FMAC2 soap_faultdetail(struct soap*);
+SOAP_FMAC1 const char* SOAP_FMAC2 soap_check_faultsubcode(struct soap*);
+SOAP_FMAC1 const char* SOAP_FMAC2 soap_check_faultdetail(struct soap*);
 SOAP_FMAC1 void SOAP_FMAC2 soap_serializeheader(struct soap*);
 SOAP_FMAC1 int SOAP_FMAC2 soap_putheader(struct soap*);
 SOAP_FMAC1 int SOAP_FMAC2 soap_getheader(struct soap*);
@@ -2303,17 +2327,13 @@ SOAP_FMAC1 char* SOAP_FMAC2 soap_putoffsets(struct soap*, const int *, int);
 SOAP_FMAC1 int SOAP_FMAC2 soap_closesock(struct soap*);
 SOAP_FMAC1 int SOAP_FMAC2 soap_force_closesock(struct soap*);
 
-SOAP_FMAC1 struct soap *SOAP_FMAC2 soap_new(void);
-SOAP_FMAC1 struct soap *SOAP_FMAC2 soap_new1(soap_mode);
-SOAP_FMAC1 struct soap *SOAP_FMAC2 soap_new2(soap_mode, soap_mode);
+SOAP_FMAC1 struct soap *SOAP_FMAC2 soap_new0(int, soap_mode, soap_mode);
 SOAP_FMAC1 void SOAP_FMAC2 soap_free(struct soap*);
 SOAP_FMAC1 struct soap *SOAP_FMAC2 soap_copy(const struct soap*);
 SOAP_FMAC1 struct soap *SOAP_FMAC2 soap_copy_context(struct soap*, const struct soap*);
 SOAP_FMAC1 void SOAP_FMAC2 soap_copy_stream(struct soap*, struct soap*);
 SOAP_FMAC1 void SOAP_FMAC2 soap_free_stream(struct soap*);
-SOAP_FMAC1 void SOAP_FMAC2 soap_init(struct soap*);
-SOAP_FMAC1 void SOAP_FMAC2 soap_init1(struct soap*, soap_mode);
-SOAP_FMAC1 void SOAP_FMAC2 soap_init2(struct soap*, soap_mode, soap_mode);
+SOAP_FMAC1 int SOAP_FMAC2 soap_init0(struct soap*, int, soap_mode, soap_mode);
 SOAP_FMAC1 void SOAP_FMAC2 soap_done(struct soap*);
 SOAP_FMAC1 void SOAP_FMAC2 soap_cleanup(struct soap*);
 SOAP_FMAC1 void SOAP_FMAC2 soap_begin(struct soap*);
@@ -2423,8 +2443,10 @@ SOAP_FMAC1 int SOAP_FMAC2 soap_recv_fault(struct soap*, int check);
 SOAP_FMAC1 void SOAP_FMAC2 soap_print_fault(struct soap*, FILE*);
 SOAP_FMAC1 void SOAP_FMAC2 soap_print_fault_location(struct soap*, FILE*);
 # ifndef WITH_LEAN
-#  ifdef __cplusplus
+#  ifndef WITH_COMPAT
+#   ifdef __cplusplus
 SOAP_FMAC1 void SOAP_FMAC2 soap_stream_fault(struct soap*, std::ostream&);
+#   endif
 #  endif
 SOAP_FMAC1 char* SOAP_FMAC2 soap_sprint_fault(struct soap*, char*, size_t);
 # endif
