@@ -49,6 +49,7 @@ A commercial use license is available from Genivia, Inc., contact@genivia.com
 #include "stdsoap2.h"
 
 SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_xsd__anyType(struct soap*, struct soap_dom_element const*);
+SOAP_FMAC3 void SOAP_FMAC4 soap_traverse_xsd__anyType(struct soap*, struct soap_dom_element*, const char*, soap_walker, soap_walker);
 SOAP_FMAC1 void SOAP_FMAC2 soap_default_xsd__anyType(struct soap*, struct soap_dom_element *);
 SOAP_FMAC3 int SOAP_FMAC4 soap_put_xsd__anyType(struct soap*, const struct soap_dom_element *, const char*, const char*);
 SOAP_FMAC1 int SOAP_FMAC2 soap_out_xsd__anyType(struct soap*, const char*, int, const struct soap_dom_element *, const char*);
@@ -56,6 +57,7 @@ SOAP_FMAC3 struct soap_dom_element * SOAP_FMAC4 soap_get_xsd__anyType(struct soa
 SOAP_FMAC1 struct soap_dom_element * SOAP_FMAC2 soap_in_xsd__anyType(struct soap*, const char*, struct soap_dom_element *, const char*);
 
 SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_xsd__anyAttribute(struct soap*, struct soap_dom_attribute const*);
+SOAP_FMAC3 void SOAP_FMAC4 soap_traverse_xsd__anyAttribute(struct soap*, struct soap_dom_attribute*, const char*, soap_walker, soap_walker);
 SOAP_FMAC1 void SOAP_FMAC2 soap_default_xsd__anyAttribute(struct soap*, struct soap_dom_attribute *);
 SOAP_FMAC3 int SOAP_FMAC4 soap_put_xsd__anyAttribute(struct soap*, const struct soap_dom_attribute *, const char*, const char*);
 SOAP_FMAC1 int SOAP_FMAC2 soap_out_xsd__anyAttribute(struct soap*, const char*, int, const struct soap_dom_attribute *, const char*);
@@ -107,6 +109,13 @@ soap_serialize_xsd__anyType(struct soap *soap, const struct soap_dom_element *no
   }
 }
 
+SOAP_FMAC3
+void
+SOAP_FMAC4
+soap_traverse_xsd__anyType(struct soap *soap, struct soap_dom_element *node, const char *s, soap_walker p, soap_walker q)
+{ 
+}
+
 /******************************************************************************/
 
 SOAP_FMAC1
@@ -116,6 +125,13 @@ soap_serialize_xsd__anyAttribute(struct soap *soap, const struct soap_dom_attrib
 {
 }
 
+SOAP_FMAC1
+void
+SOAP_FMAC2
+soap_traverse_xsd__anyAttribute(struct soap *soap, struct soap_dom_attribute *node, const char *s, soap_walker p, soap_walker q)
+{
+}
+
 /******************************************************************************/
 
 SOAP_FMAC1
@@ -862,6 +878,21 @@ soap_dom_attribute::~soap_dom_attribute()
 
 /******************************************************************************/
 
+soap_dom_attribute &soap_dom_attribute::set(const char *nstr, const char *name)
+{ this->nstr = soap_strdup(soap, nstr);
+  this->name = soap_strdup(soap, name);
+  return *this;
+}
+
+/******************************************************************************/
+
+soap_dom_attribute &soap_dom_attribute::set(const char *data)
+{ this->data = soap_strdup(soap, name);
+  return *this;
+}
+
+/******************************************************************************/
+
 soap_dom_attribute_iterator soap_dom_attribute::begin()
 { soap_dom_attribute_iterator iter(this);
   return iter;
