@@ -270,8 +270,16 @@ soap_mec_init(struct soap *soap, struct soap_mec_data *data, int alg, SOAP_MEC_K
   EVP_CIPHER_CTX_init(data->ctx);
   data->alg = alg;
   data->state = SOAP_MEC_STATE_NONE;
-  /* triple DES CBC only */
-  data->type = EVP_des_ede3_cbc();
+  if (alg & SOAP_MEC_DES_CBC)
+    data->type = EVP_des_ede3_cbc(); /* triple DES CBC */
+  else if (alg & SOAP_MEC_AES128)
+    data->type = EVP_get_cipherbyname("AES128");
+  else if (alg & SOAP_MEC_AES256)
+    data->type = EVP_get_cipherbyname("AES256");
+  else if (alg & SOAP_MEC_AES512)
+    data->type = EVP_get_cipherbyname("AES512");
+  else
+    data->type = EVP_enc_null();
   data->rest = NULL;
   data->restlen = 0;
   if (alg & SOAP_MEC_ENC)
@@ -385,7 +393,7 @@ soap_mec_begin(struct soap *soap, struct soap_mec_data *data, int alg, SOAP_MEC_
   /* save and set the engine's 'data' field to pass data to the callbacks */
   soap->data[1] = (void*)data;
   data->ctx = NULL;
-  data->type = NULL;
+  data->type = EVP_enc_null();
   data->pkey = NULL;
   data->key = NULL;
   data->buf = NULL;
@@ -652,14 +660,12 @@ soap_mec_upd_enc(struct soap *soap, struct soap_mec_data *data, const char **s,
 static int
 soap_mec_upd_dec(struct soap *soap, struct soap_mec_data *data, const char **s, size_t *n, int final)
 { const char *r = NULL;
-  const char *t;
-  size_t k, l, m;
-  int len;
-  enum SOAP_MEC_STATE state;
+  size_t k = 0, l = 0, m = 0;
+  int len = 0;
   int ok = 1;
-  if (final && data->state == SOAP_MEC_STATE_DECRYPT)
+  enum SOAP_MEC_STATE state = data->state;
+  if (final && state == SOAP_MEC_STATE_DECRYPT)
     data->state = SOAP_MEC_STATE_FINAL;
-  state = data->state;
   /* if flushing the buf, no base64-decode or decryption to do */
   if (state == SOAP_MEC_STATE_FLUSH)
   { /* old + new fit in buf? */
@@ -818,8 +824,8 @@ soap_mec_upd_dec(struct soap *soap, struct soap_mec_data *data, const char **s,
         data->bufidx = 0; /* next decoded goes to start of buf */
       break;
     case SOAP_MEC_STATE_FINAL:
-      /* we know there is enough space to flush *s and *n through the buf */
-      t = *s;
+    { /* we know there is enough space to flush *s and *n through the buf */
+      const char *t = *s;
       k = *n;
       len = 0;
       switch (data->alg & ~SOAP_MEC_STORE)
@@ -843,11 +849,12 @@ soap_mec_upd_dec(struct soap *soap, struct soap_mec_data *data, const char **s,
         memmove(data->buf + *n, t, k);
         *n += k;
       }
-      DBGLOG(TEST, SOAP_MESSAGE(fdebug, "Final state n=%lu\n", (unsigned long)*n));
       if (!(data->alg & SOAP_MEC_STORE))
         data->bufidx = 0; /* next decoded goes to start of buf */
+      DBGLOG(TEST, SOAP_MESSAGE(fdebug, "Final len=%lu\n", (unsigned long)*n));
       state = SOAP_MEC_STATE_FLUSH; /* flush data buf, if needed */
       break;
+    }
     default:
       DBGLOG(TEST, SOAP_MESSAGE(fdebug, "Error in decryption state\n"));
       return soap->error = SOAP_SSL_ERROR;
@@ -858,7 +865,9 @@ soap_mec_upd_dec(struct soap *soap, struct soap_mec_data *data, const char **s,
       DBGLOG(TEST, SOAP_MESSAGE(fdebug, "Append rest of stream %lu (%lu <= %lu)\n", data->restlen, *n, data->buflen));
     }
     else
+    { DBGLOG(TEST, SOAP_MESSAGE(fdebug, "Error in decryption state\n"));
       return soap->error = SOAP_SSL_ERROR;
+    }
   }
   data->state = state;
   return soap_mec_check(soap, data, ok, "soap_mec_upd_dec() failed");
@@ -877,10 +886,12 @@ static int
 soap_mec_check(struct soap *soap, struct soap_mec_data *data, int ok, const char *msg)
 { if (ok <= 0)
   { unsigned long r;
+    DBGLOG(TEST, SOAP_MESSAGE(fdebug, "-- MEC Error (%d)", ok));
     while ((r = ERR_get_error()))
     { ERR_error_string_n(r, soap->msgbuf, sizeof(soap->msgbuf));
-      DBGLOG(TEST, SOAP_MESSAGE(fdebug, "-- MEC Error (%d) %s: %s\n", ok, msg, soap->msgbuf));
+      DBGLOG(TEST, SOAP_MESSAGE(fdebug, " %s: \"%s\";", msg, soap->msgbuf));
     }
+    DBGLOG(TEST, SOAP_MESSAGE(fdebug, "\n"));
     /* cleanup and reset mecevp engine */
     soap_mec_cleanup(soap, data);
     return soap_set_receiver_error(soap, msg, soap->msgbuf, SOAP_SSL_ERROR);
@@ -1051,11 +1062,12 @@ soap_mec_filtersend(struct soap *soap, const char **s, size_t *n)
 }
 
 /**
-@fn int soap_mec_filtersend(struct soap *soap, const char **s, size_t *n)
+@fn int soap_mec_filterrecv(struct soap *soap, char *buf, size_t *len, size_t maxlen)
 @brief Callback to modify inbound messages by decrypting through the engine.
 @param soap context
-@param[in,out] s encrypted message, afterwards set to decrypted message
-@param[in,out] n encrypted message size, afterwards set to decrypted message size
+@param[in,out] buf encrypted message, afterwards contains decrypted content
+@param[in,out] len encrypted message size, afterwards set to decrypted content size
+@param[in] maxlen max length of allocated buf size to contain decrypted content
 @return SOAP_OK or SOAP_SSL_ERROR
 */
 static int
