@@ -1,7 +1,7 @@
 /*
 	wsddapi.c
 
-	WS-Discovery 1.1 (WSDD) plugin API
+	WS-Discovery 1.1 and 1.0 (WSDD) plugin API
 
 	See gsoap/doc/wsdd for the WSDD plugin user guide.
 
@@ -251,18 +251,71 @@ soap_wsdd_listen(serv, -1000);
 
 Again, the id and state are used to associate the asynchronously received
 ResolveMatches response that is handled by the @ref wsdd_event_ResolveMatches
-to the original request.
+for the original request.
 
 In managed mode with unicast messages (request-response messages), the @ref
 soap_wsdd_Probe and @ref soap_wsdd_Resolve are sufficient to invoke without
 setting up a listener. The event handlers are invoked when the unicast response
 message arrives.
 
-The ProbeMatches and ResolveMatches are automatically send via @ref
-soap_wsdd_listen and the event @ref wsdd_event_Probe and @ref
+In managed mode, the ProbeMatches and ResolveMatches are automatically sent via
+@ref soap_wsdd_listen and the event @ref wsdd_event_Probe and @ref
 wsdd_event_Resolve handlers. These event handlers should set the matches to be
 returned.
 
+In ad-hoc mode, ProbeMatches or ResolveMatches responses are NOT sent
+automatically. In ad-hoc mode the responses can be returned by adding code to
+the event handler or from anywhere in the main program, for example after @ref
+soap_wsdd_listen. When responses are to be returned from the event handler or
+from the main program, you should invoke @ref soap_wsdd_ProbeMatches and @ref
+soap_wsdd_ResolveMatches to explicitly send unicast messages with the match(es)
+back to the clients. The WS-Addressing ReplyTo address can be used as the
+return address (when not anonymous), or by using the peer's host information
+that is accessible in the soap->peer and soap->peerlen members. For example:
+
+@code
+char host[1024], port[16];
+getnameinfo((struct sockaddr*)&soap->peer, soap->peerlen, host, sizeof(host), port, 16, NI_DGRAM | NI_NAMEREQD | NI_NUMERICSERV);
+@endcode
+
+@section wsdd_5 Generating C++ Server Objects
+
+The WSDD library is developed to support C and C++. To support C++ server
+objects generated with soapcpp2 options -i and -j, you need to define in your
+C++ code the following wrappers (use this->soap below for soapcpp2 option -j):
+
+@code
+int wsddService::Hello(struct wsdd__HelloType *hello)
+{ return __wsdd__Hello(this, hello);
+}
+int wsddService::Bye(struct wsdd__ByeType *bye
+{ return __wsdd__Bye(this, bye);
+}
+int wsddService::Probe(struct wsdd__ProbeType *probe)
+{ return __wsdd__Probe(this, probe);
+}
+int wsddService::ProbeMatches(struct wsdd__ProbeMatchesType *matches)
+{ return __wsdd__ProbeMatches(this, matches);
+}
+int wsddService::Resolve(struct wsdd__ResolveType *resolve)
+{ return __wsdd__Resolve(this, resolve);
+}
+int wsddService::ResolveProbeMatches(struct wsdd__ResolveMatchesType *matches)
+{ return __wsdd__ResolveMatches(this, matches);
+}
+@endcode
+
+@section wsdd_6 Miscellaneous
+
+You MUST generate client-side operations that the WSDD library expects to be
+linked with, by executing:
+
+@code
+> soapcpp2 -CL -Iimport import/wsdd.h
+@endcode
+
+Then compile and link the generated soapClient.cpp code with your project.
+
 */
 
 #include "wsddapi.h"
@@ -279,7 +332,6 @@ static unsigned int soap_wsdd_InstanceId = 0;
 static const char *soap_wsdd_SequenceId = NULL;
 static unsigned int soap_wsdd_MessageNumber = 1;
 
-static int soap_wsdd_serve_request(struct soap *soap);
 static int soap_wsdd_serve___wsdd__Hello(struct soap *soap);
 static int soap_wsdd_serve___wsdd__Bye(struct soap *soap);
 static int soap_wsdd_serve___wsdd__Probe(struct soap *soap);
@@ -291,16 +343,16 @@ static void soap_wsdd_set_AppSequence(struct soap*);
 static void soap_wsdd_reset_AppSequence(struct soap*);
 static void soap_wsdd_delay(struct soap*);
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
 /******************************************************************************\
  *
  *	WS-Discovery Outbound Messages
  *
 \******************************************************************************/
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /**
 @fn int soap_wsdd_Hello(struct soap *soap, soap_wsdd_mode mode, const char *endpoint, const char *MessageID, const char *RelatesTo, const char *EndpointReference, const char *Types, const char *Scopes, const char *MatchBy, const char *XAddrs, unsigned int MetadataVersion)
 @brief TS or DP Hello message to join a network.
@@ -341,7 +393,11 @@ soap_wsdd_Hello(struct soap *soap, soap_wsdd_mode mode, const char *endpoint, co
 
   /* Hello */
   soap_default_wsdd__HelloType(soap, &req);
+#ifdef SOAP_WSA_2005
   req.wsa5__EndpointReference.Address = (char*)EndpointReference;
+#else
+  req.wsa__EndpointReference.Address = (char*)EndpointReference;
+#endif
   req.Types = (char*)Types;
   if (Scopes)
   {
@@ -398,7 +454,11 @@ soap_wsdd_Bye(struct soap *soap, soap_wsdd_mode mode, const char *endpoint, cons
 
   /* Bye */
   soap_default_wsdd__ByeType(soap, &req);
+#ifdef SOAP_WSA_2005
   req.wsa5__EndpointReference.Address = (char*)EndpointReference;
+#else
+  req.wsa__EndpointReference.Address = (char*)EndpointReference;
+#endif
   req.Types = (char*)Types;
   if (Scopes)
   {
@@ -501,8 +561,13 @@ soap_wsdd_Probe(struct soap *soap, soap_wsdd_mode mode, soap_wsdd_to to, const c
       InstanceId,
       SequenceId,
       MessageNumber, 
+#ifdef SOAP_WSA_2005
       soap->header->wsa5__MessageID,
       soap->header->wsa5__RelatesTo ? soap->header->wsa5__RelatesTo->__item : NULL,
+#else
+      soap->header->wsa__MessageID,
+      soap->header->wsa__RelatesTo ? soap->header->wsa__RelatesTo->__item : NULL,
+#endif
       res.wsdd__ProbeMatches);
   }
   return SOAP_OK;
@@ -546,7 +611,11 @@ soap_wsdd_Resolve(struct soap *soap, soap_wsdd_mode mode, soap_wsdd_to to, const
 
   /* Resolve */
   soap_default_wsdd__ResolveType(soap, &req);
+#ifdef SOAP_WSA_2005
   req.wsa5__EndpointReference.Address = (char*)EndpointReference;
+#else
+  req.wsa__EndpointReference.Address = (char*)EndpointReference;
+#endif
   if (soap_send___wsdd__Resolve(soap, endpoint, Action, &req))
     return soap->error;
 
@@ -582,8 +651,13 @@ soap_wsdd_Resolve(struct soap *soap, soap_wsdd_mode mode, soap_wsdd_to to, const
         InstanceId,
         SequenceId,
         MessageNumber, 
+#ifdef SOAP_WSA_2005
         soap->header->wsa5__MessageID,
         soap->header->wsa5__RelatesTo ? soap->header->wsa5__RelatesTo->__item : NULL,
+#else
+        soap->header->wsa__MessageID,
+        soap->header->wsa__RelatesTo ? soap->header->wsa__RelatesTo->__item : NULL,
+#endif
         res.wsdd__ResolveMatches->ResolveMatch);
   }
   return SOAP_OK;
@@ -645,7 +719,11 @@ soap_wsdd_add_ProbeMatch(struct soap *soap, struct wsdd__ProbeMatchesType *match
   /* add entry */
   m = &matches->ProbeMatch[k];
   soap_default_wsdd__ProbeMatchType(soap, m);
+#ifdef SOAP_WSA_2005
   m->wsa5__EndpointReference.Address = (char*)EndpointReference;
+#else
+  m->wsa__EndpointReference.Address = (char*)EndpointReference;
+#endif
   m->Types = (char*)Types;
   if (Scopes)
   {
@@ -737,7 +815,11 @@ soap_wsdd_ResolveMatches(struct soap *soap, const char *endpoint, const char *Me
 
   /* ResolveMatches */
   soap_default_wsdd__ResolveMatchType(soap, &match);
+#ifdef SOAP_WSA_2005
   match.wsa5__EndpointReference.Address = (char*)EndpointReference;
+#else
+  match.wsa__EndpointReference.Address = (char*)EndpointReference;
+#endif
   match.Types = (char*)Types;
   if (Scopes)
   {
@@ -818,7 +900,7 @@ soap_wsdd_listen(struct soap *soap, int timeout)
   } 
 }
 
-static int
+int
 soap_wsdd_serve_request(struct soap *soap)
 {
   soap_peek_element(soap);
@@ -834,9 +916,15 @@ soap_wsdd_serve_request(struct soap *soap)
     return soap_wsdd_serve___wsdd__Resolve(soap);
   if (!soap_match_tag(soap, soap->tag, "wsdd:ResolveMatches"))
     return soap_wsdd_serve___wsdd__ResolveMatches(soap);
+  soap->error = SOAP_NO_METHOD;
+  /* close socket unless soap->keep_alive == 1 */
   return soap_closesock(soap);
 }
 
+#ifdef __cplusplus
+}
+#endif
+
 static int
 soap_wsdd_serve___wsdd__Hello(struct soap *soap)
 {	struct __wsdd__Hello soap_tmp___wsdd__Hello;
@@ -958,10 +1046,19 @@ __wsdd__Hello(struct soap *soap, struct wsdd__HelloType *Hello)
 
   /* check WSDD */
   if (!Hello
-   || !Hello->wsa5__EndpointReference.Address)
+#ifdef SOAP_WSA_2005
+   || !Hello->wsa5__EndpointReference.Address
+#else
+   || !Hello->wsa__EndpointReference.Address
+#endif
+    )
     return soap_wsa_sender_fault(soap, "WSDD Hello incomplete", NULL);
 
+#ifdef SOAP_WSA_2005
   DBGLOG(TEST, SOAP_MESSAGE(fdebug, "EndpointReference=%s\n", Hello->wsa5__EndpointReference.Address));
+#else
+  DBGLOG(TEST, SOAP_MESSAGE(fdebug, "EndpointReference=%s\n", Hello->wsa__EndpointReference.Address));
+#endif
 
   if (soap->header->wsdd__AppSequence)
   {
@@ -977,9 +1074,15 @@ __wsdd__Hello(struct soap *soap, struct wsdd__HelloType *Hello)
     InstanceId,
     SequenceId,
     MessageNumber, 
+#ifdef SOAP_WSA_2005
     soap->header->wsa5__MessageID,
     soap->header->wsa5__RelatesTo ? soap->header->wsa5__RelatesTo->__item : NULL,
     Hello->wsa5__EndpointReference.Address,
+#else
+    soap->header->wsa__MessageID,
+    soap->header->wsa__RelatesTo ? soap->header->wsa__RelatesTo->__item : NULL,
+    Hello->wsa__EndpointReference.Address,
+#endif
     Hello->Types,
     Hello->Scopes ? Hello->Scopes->__item : NULL,
     Hello->Scopes ? Hello->Scopes->MatchBy : NULL,
@@ -1011,10 +1114,19 @@ __wsdd__Bye(struct soap *soap, struct wsdd__ByeType *Bye)
 
   /* check WSDD */
   if (!Bye
-   || !Bye->wsa5__EndpointReference.Address)
+#ifdef SOAP_WSA_2005
+   || !Bye->wsa5__EndpointReference.Address
+#else
+   || !Bye->wsa__EndpointReference.Address
+#endif
+    )
     return soap_wsa_sender_fault(soap, "WSDD Bye incomplete", NULL);
 
+#ifdef SOAP_WSA_2005
   DBGLOG(TEST, SOAP_MESSAGE(fdebug, "EndpointReference=%s\n", Bye->wsa5__EndpointReference.Address));
+#else
+  DBGLOG(TEST, SOAP_MESSAGE(fdebug, "EndpointReference=%s\n", Bye->wsa__EndpointReference.Address));
+#endif
 
   if (soap->header->wsdd__AppSequence)
   {
@@ -1030,9 +1142,15 @@ __wsdd__Bye(struct soap *soap, struct wsdd__ByeType *Bye)
     InstanceId,
     SequenceId,
     MessageNumber, 
+#ifdef SOAP_WSA_2005
     soap->header->wsa5__MessageID,
     soap->header->wsa5__RelatesTo ? soap->header->wsa5__RelatesTo->__item : NULL,
     Bye->wsa5__EndpointReference.Address,
+#else
+    soap->header->wsa__MessageID,
+    soap->header->wsa__RelatesTo ? soap->header->wsa__RelatesTo->__item : NULL,
+    Bye->wsa__EndpointReference.Address,
+#endif
     Bye->Types,
     Bye->Scopes ? Bye->Scopes->__item : NULL,
     Bye->Scopes ? Bye->Scopes->MatchBy : NULL,
@@ -1074,8 +1192,13 @@ __wsdd__Probe(struct soap *soap, struct wsdd__ProbeType *Probe)
 
   /* pass probe request on to user-defined event handler to get matches */
   mode = wsdd_event_Probe(soap,
+#ifdef SOAP_WSA_2005
     soap->header->wsa5__MessageID,
     soap->header->wsa5__ReplyTo ? soap->header->wsa5__ReplyTo->Address : NULL,
+#else
+    soap->header->wsa__MessageID,
+    soap->header->wsa__ReplyTo ? soap->header->wsa__ReplyTo->Address : NULL,
+#endif
     Probe->Types,
     Probe->Scopes ? Probe->Scopes->__item : NULL,
     Probe->Scopes ? Probe->Scopes->MatchBy : NULL,
@@ -1083,6 +1206,8 @@ __wsdd__Probe(struct soap *soap, struct wsdd__ProbeType *Probe)
 
   if (mode == SOAP_WSDD_ADHOC)
   {
+    /* event handler or main application MUST return matches independently */
+
     /* one-way HTTP(S) POST message requires an HTTP OK response message */
     if (!strncmp(soap->endpoint, "http", 4))
       return soap_send_empty_response(soap, SOAP_OK);
@@ -1143,8 +1268,13 @@ __wsdd__ProbeMatches(struct soap *soap, struct wsdd__ProbeMatchesType *ProbeMatc
     InstanceId,
     SequenceId,
     MessageNumber, 
+#ifdef SOAP_WSA_2005
     soap->header->wsa5__MessageID,
     soap->header->wsa5__RelatesTo ? soap->header->wsa5__RelatesTo->__item : NULL,
+#else
+    soap->header->wsa__MessageID,
+    soap->header->wsa__RelatesTo ? soap->header->wsa__RelatesTo->__item : NULL,
+#endif
     ProbeMatches);
 
   /* one-way HTTP(S) POST message requires an HTTP OK response message */
@@ -1172,7 +1302,12 @@ __wsdd__Resolve(struct soap *soap, struct wsdd__ResolveType *Resolve)
 
   /* check WSDD */
   if (!Resolve
-   || !Resolve->wsa5__EndpointReference.Address)
+#ifdef SOAP_WSA_2005
+   || !Resolve->wsa5__EndpointReference.Address
+#else
+   || !Resolve->wsa__EndpointReference.Address
+#endif
+    )
     return soap_wsa_sender_fault(soap, "WSDD Resolve incomplete", NULL);
 
   soap_default_wsdd__ResolveMatchesType(soap, &ResolveMatches);
@@ -1181,13 +1316,21 @@ __wsdd__Resolve(struct soap *soap, struct wsdd__ResolveType *Resolve)
 
   /* pass resolve request on to user-defined event handler */
   mode = wsdd_event_Resolve(soap,
+#ifdef SOAP_WSA_2005
     soap->header->wsa5__MessageID,
     soap->header->wsa5__ReplyTo ? soap->header->wsa5__ReplyTo->Address : NULL,
     Resolve->wsa5__EndpointReference.Address,
+#else
+    soap->header->wsa__MessageID,
+    soap->header->wsa__ReplyTo ? soap->header->wsa__ReplyTo->Address : NULL,
+    Resolve->wsa__EndpointReference.Address,
+#endif
     &match);
 
   if (mode == SOAP_WSDD_ADHOC)
   {
+    /* event handler or main application MUST return match independently */
+
     /* one-way HTTP(S) POST message requires an HTTP OK response message */
     if (!strncmp(soap->endpoint, "http", 4))
       return soap_send_empty_response(soap, SOAP_OK);
@@ -1197,10 +1340,7 @@ __wsdd__Resolve(struct soap *soap, struct wsdd__ResolveType *Resolve)
     int err;
     int (*fpost)(struct soap*, const char*, const char*, int, const char*, const char*, size_t);
     const char *MessageID = soap_wsa_rand_uuid(soap);
-    const char *Action = "http://docs.oasis-open.org/ws-dd/ns/discovery/2009/01/ProbeMatches";
-
-    /* 0..APP_MAX_DELAY ms delay */
-    soap_wsdd_delay(soap);
+    const char *Action = "http://docs.oasis-open.org/ws-dd/ns/discovery/2009/01/ResolveMatches";
 
     /* SOAP Header */
     soap_wsdd_set_AppSequence(soap);
@@ -1252,8 +1392,13 @@ __wsdd__ResolveMatches(struct soap *soap, struct wsdd__ResolveMatchesType *Resol
     InstanceId,
     SequenceId,
     MessageNumber, 
+#ifdef SOAP_WSA_2005
     soap->header->wsa5__MessageID,
     soap->header->wsa5__RelatesTo ? soap->header->wsa5__RelatesTo->__item : NULL,
+#else
+    soap->header->wsa__MessageID,
+    soap->header->wsa__RelatesTo ? soap->header->wsa__RelatesTo->__item : NULL,
+#endif
     ResolveMatches->ResolveMatch);
 
   /* one-way HTTP(S) POST message requires an HTTP OK response message */
@@ -1268,6 +1413,10 @@ __wsdd__ResolveMatches(struct soap *soap, struct wsdd__ResolveMatchesType *Resol
  *
 \******************************************************************************/
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /**
 @fn void soap_wsdd_set_InstanceId(unsigned int InstanceId)
 @brief Set the global AppSequence InstanceId that is used to populate messages
@@ -1298,6 +1447,10 @@ soap_wsdd_set_SequenceId(const char *SequenceId)
   MUTEX_UNLOCK(soap_wsdd_state);
 }
 
+#ifdef __cplusplus
+}
+#endif
+
 static void
 soap_wsdd_set_AppSequence(struct soap *soap)
 {
@@ -1328,7 +1481,3 @@ soap_wsdd_delay(struct soap *soap)
   useconds_t delay = 1000*(soap_random % SOAP_WSDD_APP_MAX_DELAY);
   usleep(delay);
 }
-
-#ifdef __cplusplus
-}
-#endif
