@@ -73,8 +73,8 @@ The signature key (private) and verification key (public) are kept in the
 plugin data, together with other info.
 */
 struct soap_wsse_data
-{ char *sigid;			/**< string with wsu:Id names to sign */
-  char *encid;			/**< string with wsu:Id names to encrypt */
+{ const char *sigid;		/**< string with wsu:Id names to sign */
+  const char *encid;		/**< string with wsu:Id names to encrypt */
   int sign_alg;			/**< The digest or signature algorithm used */
   const void *sign_key;		/**< EVP_PKEY or key string for HMAC */
   int sign_keylen;		/**< HMAC key length */
@@ -202,10 +202,10 @@ int soap_wsse_verify_done(struct soap *soap);
 size_t soap_wsse_verify_element(struct soap *soap, const char *URI, const char *tag);
 int soap_wsse_verify_body(struct soap *soap);
 int soap_wsse_set_wsu_id(struct soap *soap, const char *tags);
-int soap_wsse_sign_only(struct soap *soap, const char *ids);
-int soap_wsse_encrypt_only(struct soap *soap, const char *ids);
+int soap_wsse_sign_only(struct soap *soap, const char *tags);
 
 int soap_wsse_add_EncryptedKey(struct soap *soap, const char *URI, X509 *cert, const char *subjectkeyid);
+int soap_wsse_add_EncryptedKey_encrypt_only(struct soap *soap, const char *URI, X509 *cert, const char *subjectkeyid, const char *tags);
 int soap_wsse_verify_EncryptedKey(struct soap *soap);
 void soap_wsse_delete_EncryptedKey(struct soap *soap);
 struct xenc__EncryptedKeyType* soap_wsse_EncryptedKey(struct soap *soap);
@@ -214,6 +214,7 @@ int soap_wsse_add_EncryptedKey_DataReferenceURI(struct soap *soap, const char *U
 int soap_wsse_add_DataReferenceURI(struct soap *soap, const char *URI);
 
 int soap_wsse_encrypt_body(struct soap *soap, int alg, const void *key, int keylen);
+int soap_wsse_encrypt_only(struct soap *soap, int alg, const void *key, int keylen, const char *tags);
 int soap_wsse_encrypt(struct soap *soap, int alg, const void *key, int keylen);
 int soap_wsse_decrypt_auto(struct soap *soap, int alg, const void *key, int keylen);
 
